@@ -1,3 +1,5 @@
+// +build !windows
+
 package main
 
 import (
@@ -9,7 +11,7 @@ import (
 	"strconv"
 	"strings"
 
-	log "github.com/Sirupsen/logrus"
+	"github.com/Sirupsen/logrus"
 	"github.com/docker/docker/daemon/graphdriver/devmapper"
 	"github.com/docker/docker/pkg/devicemapper"
 )
@@ -63,7 +65,7 @@ func main() {
 
 	if *flDebug {
 		os.Setenv("DEBUG", "1")
-		log.SetLevel(log.DebugLevel)
+		logrus.SetLevel(logrus.DebugLevel)
 	}
 
 	if flag.NArg() < 1 {
@@ -73,7 +75,7 @@ func main() {
 	args := flag.Args()
 
 	home := path.Join(*root, "devicemapper")
-	devices, err := devmapper.NewDeviceSet(home, false, nil)
+	devices, err := devmapper.NewDeviceSet(home, false, nil, nil, nil)
 	if err != nil {
 		fmt.Println("Can't initialize device mapper: ", err)
 		os.Exit(1)
@@ -105,9 +107,9 @@ func main() {
 			fmt.Println("Can't get device info: ", err)
 			os.Exit(1)
 		}
-		fmt.Printf("Id: %d\n", status.DeviceId)
+		fmt.Printf("Id: %d\n", status.DeviceID)
 		fmt.Printf("Size: %d\n", status.Size)
-		fmt.Printf("Transaction Id: %d\n", status.TransactionId)
+		fmt.Printf("Transaction Id: %d\n", status.TransactionID)
 		fmt.Printf("Size in Sectors: %d\n", status.SizeInSectors)
 		fmt.Printf("Mapped Sectors: %d\n", status.MappedSectors)
 		fmt.Printf("Highest Mapped Sector: %d\n", status.HighestMappedSector)
@@ -125,7 +127,7 @@ func main() {
 
 		err = devices.ResizePool(size)
 		if err != nil {
-			fmt.Println("Error resizeing pool: ", err)
+			fmt.Println("Error resizing pool: ", err)
 			os.Exit(1)
 		}
 
