@@ -8,6 +8,7 @@ import (
 	"github.com/docker/docker/daemon/execdriver"
 )
 
-func NewDriver(root, initPath string) (execdriver.Driver, error) {
+// NewDriver returns a new native driver, called from NewDriver of execdriver.
+func NewDriver(root string, options []string) (execdriver.Driver, error) {
 	return nil, fmt.Errorf("native driver not supported on non-linux")
 }
