@@ -5,9 +5,13 @@ import (
 	"os/exec"
 )
 
+// StdConsole defines standard console operations for execdriver
 type StdConsole struct {
+	// Closers holds io.Closer references for closing at terminal close time
+	Closers []io.Closer
 }
 
+// NewStdConsole returns a new StdConsole struct
 func NewStdConsole(processConfig *ProcessConfig, pipes *Pipes) (*StdConsole, error) {
 	std := &StdConsole{}
 
@@ -17,6 +21,7 @@ func NewStdConsole(processConfig *ProcessConfig, pipes *Pipes) (*StdConsole, err
 	return std, nil
 }
 
+// AttachPipes attaches given pipes to exec.Cmd
 func (s *StdConsole) AttachPipes(command *exec.Cmd, pipes *Pipes) error {
 	command.Stdout = pipes.Stdout
 	command.Stderr = pipes.Stderr
@@ -35,12 +40,16 @@ func (s *StdConsole) AttachPipes(command *exec.Cmd, pipes *Pipes) error {
 	return nil
 }
 
+// Resize implements Resize method of Terminal interface
 func (s *StdConsole) Resize(h, w int) error {
-	// we do not need to reside a non tty
+	// we do not need to resize a non tty
 	return nil
 }
 
+// Close implements Close method of Terminal interface
 func (s *StdConsole) Close() error {
-	// nothing to close here
+	for _, c := range s.Closers {
+		c.Close()
+	}
 	return nil
 }
