@@ -30,8 +30,7 @@ func Init() bool {
 	return false
 }
 
-// Self returns the path to the current processes binary
-func Self() string {
+func naiveSelf() string {
 	name := os.Args[0]
 	if filepath.Base(name) == name {
 		if lp, err := exec.LookPath(name); err == nil {
@@ -42,7 +41,7 @@ func Self() string {
 	if absName, err := filepath.Abs(name); err == nil {
 		return absName
 	}
-	// if we coudn't get absolute name, return original
+	// if we couldn't get absolute name, return original
 	// (NOTE: Go only errors on Abs() if os.Getwd fails)
 	return name
 }
